/*
 * rhythmarty.c
 * 
 * Copyright (C) 2010 Sinew Software Systems
 * Authers: Vinod Kumar Dahiya
 *          Hemant Kumar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>

#include "artycellrendererpixbuf.h"
#include "rhythmarty.h"
#include "coverart.h"

const char*
rb_arty_get_album_first_entry_location(RBArtyPlugin *plugin,
                              const gchar *album_name)
{
	//This function wastes about 15 mb memory in 10000 calls
	
	const char *location=NULL;
	RhythmDB *db;
	g_object_get (plugin->priv->shell, "db" , &db, NULL);

	RhythmDBQueryModel *model = rhythmdb_query_model_new_empty(db);
	rhythmdb_do_full_query(db,RHYTHMDB_QUERY_RESULTS(model),
	                       RHYTHMDB_QUERY_PROP_EQUALS,
	                       RHYTHMDB_PROP_ALBUM,
	                       album_name,NULL);

	GtkTreePath *first_path = gtk_tree_path_new_from_string ("0");
	
	RhythmDBEntry *entry;
	if(rhythmdb_query_model_get_size(model))
	{
		entry= rhythmdb_query_model_tree_path_to_entry(model,first_path);
		location= rhythmdb_entry_get_string(entry,
		                                    RHYTHMDB_PROP_LOCATION);
		rhythmdb_entry_unref (entry);
	}

	gtk_tree_path_free (first_path);
	g_object_unref(model);

	return location;
}

void rb_album_view_item_activated(GtkIconView *iconview,
                                  GtkTreePath *path,
                                  gpointer     user_data)
{
	rb_arty_show_songs_clicked(NULL,user_data);
}

void rb_album_property_view_select_path(RBPropertyView *view,
                                        GtkTreePath *path)
{
	gchar *album_name;
	GList *list=NULL;
	GtkTreeModel *model;
	GtkTreeIter iter;

	model = GTK_TREE_MODEL(rb_property_view_get_model (view));
	if(gtk_tree_model_get_iter (model,&iter,path))
	{
		gtk_tree_model_get (model,&iter,RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE,
		                    &album_name,-1);

		list=g_list_append(list,album_name);
		rb_property_view_set_selection (view,list);

		g_list_free(list);
		list=NULL;
		g_free(album_name);
	}
}

void rb_arty_albumview_selection_changed(GtkIconView *iconview,
                                         RBArtyPlugin *plugin)
{
	GtkTreePath *first_path = gtk_tree_path_new_from_string ("0");
	GList *selected = gtk_icon_view_get_selected_items (iconview);

	//if none is selected select first
	if(selected == NULL)
	{
		gtk_icon_view_select_path (iconview,first_path);
		gtk_tree_path_free (first_path);
		return;
	}

	guint length = g_list_length(selected);

	//if all is selected with others unslect all, 
	//to get only all selected in next call
	if(g_list_find_custom(selected,first_path,(GCompareFunc)gtk_tree_path_compare)
	   && length>1)
	{
		gtk_icon_view_unselect_all (iconview);
		gtk_tree_path_free (first_path);
		g_list_free (selected);
		return;
	}

	//select corresponding treeview item
	GList *l=selected;
	while(l)
	{
		rb_album_property_view_select_path(plugin->priv->albums_prop_view,
		                                   l->data);
		l=l->next;
	}

	gtk_tree_path_free (first_path);
	g_list_free (selected);
}

static void
rbarty_album_view_cell_data_func (GtkCellLayout *column,
				 GtkCellRenderer *renderer,
				 GtkTreeModel *tree_model,
				 GtkTreeIter *iter,
				 RBArtyPlugin *plugin)
{
	char *title;
	char *str;
	guint number;

	gtk_tree_model_get (GTK_TREE_MODEL (tree_model), iter,
			    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title,
			    RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER, &number, -1);


	str = g_strdup_printf ("<b>%s</b>\n<i>%d %s</i>", title, number, _("songs"));
	
	g_object_set (G_OBJECT (renderer), "markup", str,NULL);
	
	g_free (str);
	g_free (title);
}

gboolean 
start_fetching_covers(gpointer data)
{
	RBArtyPlugin *plugin=data;

	rb_arty_get_covers_clicked(NULL,plugin);
	plugin->priv->not_first_time = TRUE;

	plugin->priv->first_time_timeout_id = 0;
	return FALSE;
}

gboolean 
idle_populate(gpointer data)
{
	
	RBArtyPlugin *plugin=data;

	gtk_icon_view_set_model (GTK_ICON_VIEW(plugin->priv->album_view),
	                         GTK_TREE_MODEL(plugin->priv->album_model));
	
	gtk_icon_view_set_markup_column (GTK_ICON_VIEW(plugin->priv->album_view),
	                                 RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE);

	//set markup col properties
	GtkCellRenderer *text_rend;
	GList *cell_list = 
		gtk_cell_layout_get_cells (GTK_CELL_LAYOUT(plugin->priv->album_view));
	GList *l=cell_list;
	while(l)
	{
		if(GTK_IS_CELL_RENDERER_TEXT(l->data))
		{
			text_rend = l->data;

			gtk_cell_renderer_set_fixed_size (text_rend,
			                                  plugin->priv->coverart_size,-1);
			//gtk_cell_renderer_set_alignment (text_rend,0.5,0.0);
			g_object_set(text_rend,"wrap-width",plugin->priv->coverart_size,
			             "wrap-mode",PANGO_WRAP_WORD_CHAR,NULL);
			gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT(plugin->priv->album_view),
			                                    text_rend,
			                                    (GtkCellLayoutDataFunc) rbarty_album_view_cell_data_func,
			                                    plugin, NULL);
			break;
		}

		l=l->next;
	}
	g_list_free (cell_list);
	
	GtkCellRenderer *rend = arty_cell_renderer_pixbuf_new ();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(plugin->priv->album_view),rend,FALSE);
	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT(plugin->priv->album_view),rend,
	                               "filename",RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE);
	g_object_set(rend,"dir",plugin->priv->cache_dir,
	             "size",plugin->priv->coverart_size,
	             "failed-pixbuf",plugin->priv->generic_pixbuf,NULL);
	plugin->priv->pixbuf_rend = rend;
	
	//connect signals now
	g_signal_connect(G_OBJECT(plugin->priv->album_view),"selection-changed",
	                 G_CALLBACK(rb_arty_albumview_selection_changed),plugin);
	g_signal_connect(G_OBJECT(plugin->priv->album_view),"item-activated",
	                 G_CALLBACK(rb_album_view_item_activated),plugin);

	//if it is firsttime run art fetching
	if(!plugin->priv->not_first_time)
	{
		if(gtk_tree_model_iter_n_children(plugin->priv->album_model,NULL)>1)
		{
			rb_arty_get_covers_clicked(NULL,plugin);
			plugin->priv->not_first_time = TRUE;
		}
		else
		{
			plugin->priv->first_time_timeout_id = 
				g_timeout_add (5000,start_fetching_covers,
				               plugin);
		}
	}

	return FALSE;
}

void rb_arty_update_cover_size (RBArtyPlugin *plugin)
{
	g_object_unref (plugin->priv->generic_pixbuf);
	
	gchar *filename;
	filename = g_strconcat(PLUGINDIR,"/rhythmarty/","generic.png",NULL);
	plugin->priv->generic_pixbuf = 
		gdk_pixbuf_new_from_file_at_size (filename,plugin->priv->coverart_size,
		                                  plugin->priv->coverart_size,NULL);
	g_free(filename);
	
	gtk_icon_view_set_item_width (GTK_ICON_VIEW(plugin->priv->album_view),
	                              plugin->priv->coverart_size);

	g_object_set(plugin->priv->pixbuf_rend,"size",plugin->priv->coverart_size,
	             "failed-pixbuf",plugin->priv->generic_pixbuf,NULL);
	
}

GtkWidget *rb_arty_create_album_view(RBArtyPlugin *plugin,
                                     RhythmDBPropertyModel *prop_model)
{
	GdkPixbuf *generic_pixbuf;

	gchar *filename;
	filename = g_strconcat(PLUGINDIR,"/rhythmarty/","generic.png",NULL);
	plugin->priv->generic_pixbuf = 
		gdk_pixbuf_new_from_file_at_size (filename,plugin->priv->coverart_size,
		                                  plugin->priv->coverart_size,NULL);
	g_free(filename);
	
	plugin->priv->album_model = GTK_TREE_MODEL(prop_model);

	

	GtkWidget *iconview = gtk_icon_view_new();
	
	gtk_icon_view_set_item_width (GTK_ICON_VIEW(iconview),plugin->priv->coverart_size);
	
	if(!plugin->priv->use_gtk_colors)
	{
		//use selected colors
		gtk_widget_modify_base (iconview,
		                        GTK_STATE_NORMAL,
		                        &plugin->priv->base_color);
		gtk_widget_modify_text (iconview,
		                        GTK_STATE_NORMAL,
		                        &plugin->priv->text_color);
		gtk_widget_modify_base (iconview,
		                        GTK_STATE_SELECTED,
		                        &plugin->priv->selection_color);
		gtk_widget_modify_base (iconview,
		                        GTK_STATE_ACTIVE,
		                        &plugin->priv->selection_color);
		gtk_widget_modify_text (iconview,
		                        GTK_STATE_SELECTED,
		                        &plugin->priv->selection_text_color);
		gtk_widget_modify_text (iconview,
		                        GTK_STATE_ACTIVE,
		                        &plugin->priv->selection_text_color);
	}
	
	gtk_icon_view_set_item_padding (GTK_ICON_VIEW(iconview),10);
	gtk_icon_view_set_spacing (GTK_ICON_VIEW(iconview),3);
	gtk_icon_view_set_row_spacing (GTK_ICON_VIEW(iconview),10);
	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW(iconview),20);
	gtk_icon_view_set_margin (GTK_ICON_VIEW(iconview),10);
	gtk_icon_view_set_selection_mode (GTK_ICON_VIEW(iconview),
	                                  GTK_SELECTION_MULTIPLE);

	g_idle_add (idle_populate,plugin);
	
	return iconview;
}

void 
rb_arty_view_entry_selected(RBPropertyView *view,
                            gchar          *name,
                            gpointer        user_data) 
{
	GList *slected = rb_property_view_get_selection(view);
	rb_property_view_set_selection(RB_PROPERTY_VIEW(user_data),slected);
	g_list_free(slected);          
}

void 
rb_arty_view_entries_selected(RBPropertyView *view,
                              gpointer        properties,
                              gpointer        user_data) 
{
	GList *slected = rb_property_view_get_selection(view);
	rb_property_view_set_selection(RB_PROPERTY_VIEW(user_data),slected);
	g_list_free(slected);          
}

void 
rb_arty_create_widgets(RBArtyPlugin *plugin,
                  RBShell *shell)
{
	//create source header buttons first
	GtkWidget *hbox, *image, *label;

	hbox = gtk_hbox_new (FALSE,5);
	image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD,GTK_ICON_SIZE_SMALL_TOOLBAR);
	//label = gtk_label_new (_("Songs"));
	//gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE,2);
	gtk_box_pack_start (GTK_BOX(hbox),image,FALSE,FALSE,2);
	
	plugin->priv->show_songs = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER(plugin->priv->show_songs),hbox);
	gtk_button_set_relief (GTK_BUTTON(plugin->priv->show_songs),GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text (plugin->priv->show_songs,
	                             _("Show songs list"));

	hbox = gtk_hbox_new (FALSE,5);
	image = gtk_image_new_from_stock (GTK_STOCK_GO_BACK,GTK_ICON_SIZE_SMALL_TOOLBAR);
	label = gtk_label_new (_("Albums"));
	gtk_box_pack_start (GTK_BOX(hbox),image,FALSE,FALSE,2);
	gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE,2);

	plugin->priv->show_albums = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER(plugin->priv->show_albums),hbox);
	gtk_button_set_relief (GTK_BUTTON(plugin->priv->show_albums),GTK_RELIEF_HALF);
	gtk_widget_set_tooltip_text (plugin->priv->show_albums,
	                             _("Show albums"));

	//add buttons to header
	GtkWidget *entry;
	RBSourceHeader *header;
	g_object_get (shell, "source-header" , &header, NULL);
	header->priv = RB_SOURCE_HEADER_GET_PRIVATE(header);

	gtk_box_pack_start (GTK_BOX(header->priv->search),plugin->priv->show_songs,FALSE,FALSE,0);
	gtk_box_reorder_child (GTK_BOX(header->priv->search),plugin->priv->show_songs,0);
	gtk_box_pack_start (GTK_BOX(header->priv->search),plugin->priv->show_albums,FALSE,FALSE,0);
	gtk_box_reorder_child (GTK_BOX(header->priv->search),plugin->priv->show_albums,1);
	
	gtk_widget_hide_all (plugin->priv->show_songs);
	gtk_widget_hide_all (plugin->priv->show_albums);

	//create panes
	plugin->priv->main_widget = gtk_vbox_new (FALSE,0);
	plugin->priv->hpane = gtk_hpaned_new ();
	plugin->priv->vpane = gtk_vpaned_new ();

	gtk_paned_pack1(GTK_PANED(plugin->priv->hpane),plugin->priv->vpane,FALSE,TRUE);
	gtk_box_pack_start (GTK_BOX(plugin->priv->main_widget),
	                    plugin->priv->hpane,TRUE,TRUE,0);

	//cover fetching widgets
	plugin->priv->fetching_box = gtk_hbox_new (FALSE,0);
	plugin->priv->fetching_progress = gtk_progress_bar_new ();
	plugin->priv->fetching_cancel = gtk_button_new();
	gtk_container_add (GTK_CONTAINER(plugin->priv->fetching_cancel),
	                   gtk_image_new_from_stock (GTK_STOCK_CANCEL,GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX(plugin->priv->fetching_box),
	                    plugin->priv->fetching_progress,TRUE,TRUE,2);
	gtk_box_pack_start (GTK_BOX(plugin->priv->fetching_box),
	                    plugin->priv->fetching_cancel,FALSE,FALSE,2);
	g_object_ref(plugin->priv->fetching_box);
	
	//gtk_box_pack_start (GTK_BOX(plugin->priv->main_widget),
	  //                  plugin->priv->fetching_box,FALSE,FALSE,2);
	
	//create views
	RhythmDB *db;
	g_object_get (shell, "db" , &db, NULL);

	GList *views = rb_source_get_property_views (plugin->priv->library_source);

	RBPropertyView *prop_view;
	RhythmDBPropertyModel *model;
	RBPropertyView *view;

	prop_view = views->data;
	model = rb_property_view_get_model(prop_view);
	view = rb_property_view_new(db,RHYTHMDB_PROP_GENRE,_("Genre"));
	rb_property_view_set_selection_mode(view,GTK_SELECTION_MULTIPLE);
	rb_property_view_set_model (view,model);
	gtk_paned_pack1(GTK_PANED(plugin->priv->vpane),GTK_WIDGET(view),FALSE,TRUE);
	plugin->priv->genre_view = GTK_WIDGET(view);
	g_signal_connect(G_OBJECT(view),"property-selected",
	                 G_CALLBACK(rb_arty_view_entry_selected),prop_view);
	g_signal_connect(G_OBJECT(view),"properties-selected",
	                 G_CALLBACK(rb_arty_view_entries_selected),prop_view);

	views = g_list_next(views);

	prop_view = views->data;
	model = rb_property_view_get_model(prop_view);
	view = rb_property_view_new(db,RHYTHMDB_PROP_ARTIST,_("Artists"));
	rb_property_view_set_selection_mode(view,GTK_SELECTION_MULTIPLE);
	rb_property_view_set_model (view,model);
	gtk_paned_pack2(GTK_PANED(plugin->priv->vpane),GTK_WIDGET(view),FALSE,TRUE);
	plugin->priv->artist_view = GTK_WIDGET(view);
	g_signal_connect(G_OBJECT(view),"property-selected",
	                 G_CALLBACK(rb_arty_view_entry_selected),prop_view);
	g_signal_connect(G_OBJECT(view),"properties-selected",
	                 G_CALLBACK(rb_arty_view_entries_selected),prop_view);

	views = g_list_next(views);

	prop_view = views->data;
	plugin->priv->albums_prop_view = prop_view;
	model = rb_property_view_get_model(prop_view);
	plugin->priv->album_view=rb_arty_create_album_view(plugin,model);
	
	GtkWidget *scroll = gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
	                               GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
	                                    GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER(scroll),plugin->priv->album_view);
	
	gtk_paned_pack2(GTK_PANED(plugin->priv->hpane),
	                GTK_WIDGET(scroll),FALSE,TRUE);
}

